==> json_path-0.4.3.tar.gz: README.md <==
# [JSONPath] in Dart
[![Pub Package](https://img.shields.io/pub/v/json_path.svg)](https://pub.dev/packages/json_path)
[![Build Status](https://github.com/f3ath/jessie/actions/workflows/dart.yml/badge.svg?branch=master)](https://github.com/f3ath/jessie/actions/workflows/dart.yml)
[![GitHub Issues](https://img.shields.io/github/issues/f3ath/jessie.svg)](https://github.com/f3ath/jessie/issues)
[![GitHub Forks](https://img.shields.io/github/forks/f3ath/jessie.svg)](https://github.com/f3ath/jessie/network)
[![GitHub Stars](https://img.shields.io/github/stars/f3ath/jessie.svg)](https://github.com/f3ath/jessie/stargazers)
[![GitHub License](https://img.shields.io/badge/license-MIT-blue.svg)](https://raw.githubusercontent.com/f3ath/jessie/master/LICENSE)

```dart
import 'dart:convert';

import 'package:json_path/json_path.dart';

void main() {
  final json = jsonDecode('''
{
  "store": {
    "book": [
      {
        "category": "reference",
        "author": "Nigel Rees",
        "title": "Sayings of the Century",
        "price": 8.95
      },
      {
        "category": "fiction",
        "author": "Evelyn Waugh",
        "title": "Sword of Honour",
        "price": 12.99
      },
      {
        "category": "fiction",
        "author": "Herman Melville",
        "title": "Moby Dick",
        "isbn": "0-553-21311-3",
        "price": 8.99
      },
      {
        "category": "fiction",
        "author": "J. R. R. Tolkien",
        "title": "The Lord of the Rings",
        "isbn": "0-395-19395-8",
        "price": 22.99
      }
    ],
    "bicycle": {
      "color": "red",
      "price": 19.95
    }
  }
}  
  ''');

  final prices = JsonPath(r'$..price');

  print('All prices in the store:');

  /// The following code will print:
  ///
  /// $['store']['book'][0]['price']:	8.95
  /// $['store']['book'][1]['price']:	12.99
  /// $['store']['book'][2]['price']:	8.99
  /// $['store']['book'][3]['price']:	22.99
  /// $['store']['bicycle']['price']:	19.95
  prices
      .read(json)
      .map((match) => '${match.path}:\t${match.value}')
      .forEach(print);
}
```

## Limitations
This library is intended to match the [original implementations], although filtering expressions (like `$..book[?(@.price < 10)]`) 
support is limited and may not always produce the results that you expect. The reason is the
difference between the type systems of Dart and JavaScript/PHP. Dart is strictly typed and does not support `eval()`,
so the expressions have to be parsed and evaluated by the library itself. Implementing complex logic this way would
create a performance overhead which I prefer to avoid.

## Callback filters
If there is a real need for complex filtering, you may use Dart-native callbacks. The syntax, _which is of course my own
invention and has nothing to do with the "official" JSONPath_, is the following:
```dart
/// Selects all elements with price under 20
final path = JsonPath(r'$.store..[?discounted]', filters: {
'discounted': (match) =>
    match.value is Map && match.value['price'] is num && match.value['price'] < 20
});
``` 
The filtering callbacks may be passed to the `JSONPath` constructor or to the `.read()` method. The callback name
must be specified in the square brackets and prefixed by `?`. It also must be alphanumeric.

## Data manipulation
Each `JsonPathMatch` produced by the `.read()` method contains the `.pointer` property which is a valid [JSON Pointer]
and can be used to write/append/remove the referenced value. If you want the data manipulation only, 
consider the [JSON Pointer implementation].

## References
- [Standard development](https://github.com/ietf-wg-jsonpath/draft-ietf-jsonpath-base)
- [Feature comparison matrix](https://cburgmer.github.io/json-path-comparison/)

[JSONPath]: https://goessner.net/articles/JsonPath/
[JSON Pointer]: https://datatracker.ietf.org/doc/html/rfc6901
[JSON Pointer implementation]: https://pub.dev/packages/rfc_6901
[original implementations]: https://goessner.net/articles/JsonPath/index.html#e4

==> json_path-0.5.0-dev.2.tar.gz: README.md <==
# [JSONPath] in Dart
[![Pub Package](https://img.shields.io/pub/v/json_path.svg)](https://pub.dev/packages/json_path)
[![Build Status](https://github.com/f3ath/jessie/actions/workflows/dart.yml/badge.svg?branch=master)](https://github.com/f3ath/jessie/actions/workflows/dart.yml)
[![GitHub Issues](https://img.shields.io/github/issues/f3ath/jessie.svg)](https://github.com/f3ath/jessie/issues)
[![GitHub Forks](https://img.shields.io/github/forks/f3ath/jessie.svg)](https://github.com/f3ath/jessie/network)
[![GitHub Stars](https://img.shields.io/github/stars/f3ath/jessie.svg)](https://github.com/f3ath/jessie/stargazers)
[![GitHub License](https://img.shields.io/badge/license-MIT-blue.svg)](https://raw.githubusercontent.com/f3ath/jessie/master/LICENSE)

```dart
import 'dart:convert';

import 'package:json_path/json_path.dart';

void main() {
  final json = jsonDecode('''
{
  "store": {
    "book": [
      {
        "category": "reference",
        "author": "Nigel Rees",
        "title": "Sayings of the Century",
        "price": 8.95
      },
      {
        "category": "fiction",
        "author": "Evelyn Waugh",
        "title": "Sword of Honour",
        "price": 12.99
      },
      {
        "category": "fiction",
        "author": "Herman Melville",
        "title": "Moby Dick",
        "isbn": "0-553-21311-3",
        "price": 8.99
      },
      {
        "category": "fiction",
        "author": "J. R. R. Tolkien",
        "title": "The Lord of the Rings",
        "isbn": "0-395-19395-8",
        "price": 22.99
      }
    ],
    "bicycle": {
      "color": "red",
      "price": 19.95
    }
  }
}  
  ''');

  final prices = JsonPath(r'$..price');

  print('All prices in the store:');

  /// The following code will print:
  ///
  /// $['store']['book'][0]['price']:	8.95
  /// $['store']['book'][1]['price']:	12.99
  /// $['store']['book'][2]['price']:	8.99
  /// $['store']['book'][3]['price']:	22.99
  /// $['store']['bicycle']['price']:	19.95
  prices
      .read(json)
      .map((node) => '${node.path}:\t${node.value}')
      .forEach(print);
}
```

## Limitations
This library follows the [JsonPath] internet draft specification. Since the spec itself is 
an evolving document, this implementation may lag behind, and some features may not be implemented in-full.
Please refer to the tests (there are hundreds of them, including the [CTS]) to see what is supported.


## Data manipulation
Each `Node` produced by the `.read()` method contains the `.pointer` property which is a valid [JSON Pointer]
and can be used to alter the referenced value. If you only need to manipulate JSON data, 
check out my [JSON Pointer implementation].

## User-defined functions
The JSONPath parser may be extended by user-defined functions. The user-defined functions
take precedence over the built-in ones which are defined by the standard. The functions
are polymorphic: you may define functions with the same name, and the parser will pick the first
of them which fits the use case (the return type, the number and the type or arguments).

## References
- [Standard development](https://github.com/ietf-wg-jsonpath/draft-ietf-jsonpath-base)
- [Feature comparison matrix](https://cburgmer.github.io/json-path-comparison/)

[CTS]: https://github.com/jsonpath-standard/jsonpath-compliance-test-suite
[JSONPath]: https://datatracker.ietf.org/wg/jsonpath/documents/
[JSON Pointer]: https://datatracker.ietf.org/doc/html/rfc6901
[JSON Pointer implementation]: https://pub.dev/packages/rfc_6901
